var Class = require('../class/Class.js').Class;

var RequestProcessor = require('../net/RequestProcessor.js').RequestProcessor;

var JsonRpcAbstract = require('../jsonrpc/JsonRpcAbstract.js').JsonRpcAbstract;
var JsonRpcRequest = require('../jsonrpc/JsonRpcRequest.js').JsonRpcRequest;
var JsonRpcResponse = require('../jsonrpc/JsonRpcResponse.js').JsonRpcResponse;
var JsonRpcMethod = require('../jsonrpc/JsonRpcMethod.js').JsonRpcMethod;

var Types = require('../common/Types.js');

var BinaryData = require('../data/BinaryData.js').BinaryData;

var JsonRpcRequestProcessor = function(){
	this.Extends = RequestProcessor;

	this._request;
	this._response;
	this._jsonRpcRequest;

	this.process = function(request, response){
	    console.time('  -- REQUEST TIME');
		var that = this, method, methodClass;

		this._request = request;
		this._response = response;

		try {
			this._jsonRpcRequest = new JsonRpcRequest(request.getBody().toBuffer().toString());
		} catch (e) {
            console.log(e);
			if (e == JsonRpcAbstract.Exceptions.INPROPER_JSON) {
				this._throwError(JsonRpcRequestProcessor.Errors.PARSE_ERROR, e);
			} else if (e == JsonRpcAbstract.Exceptions.INVALID_JSON_RPC) {
				this._throwError(JsonRpcRequestProcessor.Errors.INVALID_REQUEST, e);
			} else {
				this._throwError(JsonRpcRequestProcessor.Errors.INTERNAL_ERROR, e)
			}

			return;
		}

		methodClass = this.server.getMethod(this._jsonRpcRequest.getMethod());

		if (!methodClass) {
			this._throwError(JsonRpcRequestProcessor.Errors.METHOD_NOT_FOUND, {});
			return;
		}

		method = new methodClass(request, response, this.server);

		if (!Types.isInstanceOf(method, JsonRpcMethod)) {
			this._throwError(JsonRpcRequestProcessor.Errors.METHOD_NOT_FOUND, e);
			return;
		}

		//FIXME: tutaj trzebaby sie zastanowic jak to zrobic bo raz przekazujemy Exception a raz ErrorEvent
		method.addEventListener(JsonRpcMethod.Event.WRONG_PARAMS, this._handleError, this);
		method.addEventListener(JsonRpcMethod.Event.ERROR, this._handleError, this)
		method.addEventListener(JsonRpcMethod.Event.OK, this._success, this);
        
        console.log("  -- Executing method: " + this._jsonRpcRequest.getMethod());
        
		try {
			method.execute(this._jsonRpcRequest.getParams());
		} catch (e) {
			this._throwError(JsonRpcRequestProcessor.Errors.INTERNAL_ERROR, e);
		}
	};

	this._success = function (e) {
		if (e.data) {
			var data = e.data;

			if (!Types.isInstanceOf(e.data, JsonRpcResponse)) {
				data = new JsonRpcResponse();
				data.setResult(e.data);
			}

			data.setId(this._jsonRpcRequest.getId());
			
			this._response.setStatusCode(200);
			this._response.setHeaders({
				'Content-Type': 'application/json; charset=utf8',
				'Access-Control-Allow-Origin': '*'
			}, true);
			this._response.setBody(new BinaryData(data.toString(), BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
		}

		this._response.send();
        console.timeEnd('  -- REQUEST TIME');
        console.log('-- request ended');
	};

	this._handleError = function (e) {
        
        console.log('  -- Handling ERROR event...');
		if (e.type == JsonRpcMethod.Event.WRONG_PARAMS) {
			this._throwError(JsonRpcRequestProcessor.Errors.INVALID_PARAMS, e);
		} else if (e.type == JsonRpcMethod.Event.ERROR) {
			this._throwError(e.code, e);
		} else {
			this._throwError(JsonRpcRequestProcessor.Errors.INTERNAL_ERROR, e);
		}
	};

	this._throwError = function (code, exception) {
        
		var jsonRpcResponse = new JsonRpcResponse();
		jsonRpcResponse.setId(this._jsonRpcRequest ? this._jsonRpcRequest.getId() : "undefined");
		//TODO: tutaj if czy kod bledu jest z przedzialu zastrzezonych czy nie
		//jak z zastrzezonych to rzucamy standardowy message jak nie to bierzemy z expcetion

		if (Types.isObject(code)) {
			jsonRpcResponse.setError(parseInt(code.code, 10), code.message, exception.data ? exception.data : null);
		} else {
			jsonRpcResponse.setError(parseInt(code, 10), exception.message, exception.data ? exception.data : null);
		}
        

		this._response.setStatusCode(200);
		this._response.setHeaders({
			'Content-Type': 'application/json; charset=utf8',
			'Access-Control-Allow-Origin': '*'
		});
		this._response.setBody(new BinaryData(jsonRpcResponse.toString(), BinaryData.Encoding.TEXT, BinaryData.CharacterEncoding.UTF8));
		this._response.send();
        console.timeEnd('  -- REQUEST TIME');
        console.log('-- request ended');
	};
};

JsonRpcRequestProcessor = new Class(new JsonRpcRequestProcessor());

JsonRpcRequestProcessor.Errors = {};
JsonRpcRequestProcessor.Errors.PARSE_ERROR = { code: -32700, message: 'Parse error' };
JsonRpcRequestProcessor.Errors.INVALID_REQUEST = { code: -32600, message: 'Invalid Request' };
JsonRpcRequestProcessor.Errors.METHOD_NOT_FOUND = { code: -32601, message: 'Method not found' };
JsonRpcRequestProcessor.Errors.INVALID_PARAMS = { code: -32602, message: 'Invalid params' };
JsonRpcRequestProcessor.Errors.INTERNAL_ERROR = { code: -32603, message: 'Internal error' };


exports.JsonRpcRequestProcessor = JsonRpcRequestProcessor;
